#include "stm32f4xx.h"
#include "stm32f4xx_gpio.h"
#include "stm32f4xx_rcc.h"
#include "stm32f4xx_spi.h"

#include "main.h"    //     delay_ms(unsigned int ms); -   
#include "lcd7735.h"

#ifndef LCD_SOFT_RESET  //   
//   LCD_RST
#define LCD_RST1  GPIO_SetBits  (LCD_GPIO, LCD_RST_PIN)
#define LCD_RST0  GPIO_ResetBits(LCD_GPIO, LCD_RST_PIN)
#endif

//   LCD_DC
#define LCD_DC1   GPIO_SetBits  (LCD_GPIO, LCD_A0_PIN)
#define LCD_DC0   GPIO_ResetBits(LCD_GPIO, LCD_A0_PIN)
//   LCD_CS
#define LCD_CS1   GPIO_SetBits  (LCD_GPIO, LCD_CSE_PIN)
#define LCD_CS0   GPIO_ResetBits(LCD_GPIO, LCD_CSE_PIN)

#ifndef LCD_TO_SPI2
//   LCD_SCK
#define LCD_SCK1   GPIO_SetBits(GPIOB, LCD_SCK_PIN)
#define LCD_SCK0   GPIO_ResetBits(GPIOB, LCD_SCK_PIN)
//   LCD_MOSI
#define LCD_MOSI1   GPIO_SetBits(GPIOB, LCD_SDA_PIN)
#define LCD_MOSI0   GPIO_ResetBits(GPIOB, LCD_SDA_PIN)
#endif

//  \  
void lcd7735_senddata(unsigned char data) {

#ifdef LCD_TO_SPI2
	SPI_I2S_SendData(SPI2, data);
	while(SPI_I2S_GetFlagStatus(SPI2, SPI_I2S_FLAG_TXE) == RESET);
#else
	unsigned char i;
	for(i=0; i<8; i++) {
		if (data & 0x80) LCD_MOSI1;
		            else LCD_MOSI0;
		data = data<<1;
		LCD_SCK0;
		LCD_SCK1;
	}
#endif
}

//    
void lcd7735_sendCmd(unsigned char cmd) {
	LCD_DC0;
	lcd7735_senddata(cmd);
}

//    
void lcd7735_sendData(unsigned char data) {
	LCD_DC1;
	lcd7735_senddata(data);
}

//    
void lcd7735_pix(unsigned char X, unsigned char Y, unsigned int color) {
	lcd7735_sendCmd(0x2A);
	lcd7735_sendData(0x00);
	lcd7735_sendData(X);

	lcd7735_sendCmd(0x2B);
	lcd7735_sendData(0x00);
	lcd7735_sendData(Y);

	lcd7735_sendCmd(0x2C);
	lcd7735_sendData((color & 0xFF00)>>8);
	lcd7735_sendData(color & 0x00FF);
}

// 
void lcd7735_init(void) {
	GPIO_InitTypeDef GPIO_InitStructure;
    RCC_AHB1PeriphClockCmd(LCD_AHB1_GPIO, ENABLE); //  GPIOB

#ifdef LCD_TO_SPI2  //   SPI2

	SPI_InitTypeDef  SPI_InitStructure;
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_SPI2, ENABLE);  //  SPI2
	//  SCK (Pin_13), MOSI (Pin_15)  SPI2
	GPIO_PinAFConfig(GPIOB, GPIO_PinSource13, GPIO_AF_SPI2);   // SPI2_CLK
	GPIO_PinAFConfig(GPIOB, GPIO_PinSource15, GPIO_AF_SPI2);   // SPI2_MOSI

	GPIO_InitStructure.GPIO_Pin 			= GPIO_Pin_13 | GPIO_Pin_15;
	GPIO_InitStructure.GPIO_Mode 			= GPIO_Mode_AF;
	GPIO_InitStructure.GPIO_Speed 			= GPIO_Speed_100MHz;
	GPIO_InitStructure.GPIO_OType 			= GPIO_OType_PP;
	GPIO_InitStructure.GPIO_PuPd  			= GPIO_PuPd_UP;
	GPIO_Init(GPIOB, &GPIO_InitStructure);

	SPI_DeInit(SPI2); //   SPI2   
	SPI_InitStructure.SPI_Mode        		= SPI_Mode_Master;
	SPI_InitStructure.SPI_Direction   		= SPI_Direction_1Line_Tx;
	SPI_InitStructure.SPI_DataSize 			= SPI_DataSize_8b;
	SPI_InitStructure.SPI_CPOL 				= SPI_CPOL_Low;
	SPI_InitStructure.SPI_CPHA 				= SPI_CPHA_1Edge;
	SPI_InitStructure.SPI_NSS 				= SPI_NSS_Soft;
	SPI_InitStructure.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_2;
	SPI_InitStructure.SPI_FirstBit          = SPI_FirstBit_MSB;

	SPI_Init(SPI2, &SPI_InitStructure);
	SPI_Cmd(SPI2, ENABLE);


#ifndef	LCD_SOFT_RESET
	//  CS (Pin_10), DC (Pin_12), RST (Pin_14)
	GPIO_InitStructure.GPIO_Pin  			=LCD_CSE_PIN | LCD_A0_PIN | LCD_RST_PIN;
#else
	GPIO_InitStructure.GPIO_Pin  			=LCD_CSE_PIN | LCD_A0_PIN ;
#endif

#else //     SPI

#ifndef	LCD_SOFT_RESET
    GPIO_InitStructure.GPIO_Pin  			=LCD_CSE_PIN | LCD_A0_PIN | LCD_SCK_PIN | LCD_RST_PIN | LCD_SDA_PIN;
#else
    GPIO_InitStructure.GPIO_Pin  			=LCD_CSE_PIN | LCD_A0_PIN | LCD_SCK_PIN | LCD_SDA_PIN;
#endif

    LCD_SCK0;
#endif

    GPIO_InitStructure.GPIO_Mode 			=GPIO_Mode_OUT;
	GPIO_InitStructure.GPIO_OType			=GPIO_OType_PP;
	GPIO_InitStructure.GPIO_PuPd 			=GPIO_PuPd_UP;
	GPIO_InitStructure.GPIO_Speed			=GPIO_Speed_100MHz;
	GPIO_Init(LCD_GPIO, &GPIO_InitStructure);

	LCD_CS0;            // CS=0  -     

	//  
#ifdef	LCD_SOFT_RESET
	lcd7735_sendCmd(0x01); //  
#else
	LCD_RST0;                   // RST=0 -   
    delay_ms(LCD_RST_DLY);		// 
	LCD_RST1;                   // RST=1
#endif
	delay_ms(LCD_RST_DLY);		// 

	//   
	lcd7735_sendCmd(0x11);      //     -   

	delay_ms(LCD_RST_DLY);		// 

	lcd7735_sendCmd (0x3A); //  :
	lcd7735_sendData(0x05); //             16 
	lcd7735_sendCmd (0x29); //  

	//  
	unsigned char y;
	unsigned char x;
    //    
	for (y=0;y<160;y++)
		for (x=0;x<128;x++) {
			unsigned int color=0x0;
			if (y<20) color=0xFC00; // 
			else  {
				if (y<40) color=0x02E0; // 
			else { if (y<60) color=0x001F; } // 
			}
			lcd7735_pix(x, y, color);
		}
	LCD_CS1;
}
